@@ -6,22 +6,22 @@ test_that("fast exchange and exchange functions yield the same results - kmeans"
   # for K = 2
   clusters <- rep(1:2, 5)
   features <- matrix(rnorm(20), ncol = 2)
-  ac <- anticlustering(features, K = clusters, objective = variance_objective)
-  ac_fast <- fast_anticlustering(features, clusters)
+  ac <- anticlustering(features, K = clusters, objective = "variance")
+  ac_fast <- fast_anticlustering(features, clusters, exchange_partners = rep(list(1:10), 10))
   expect_equal(all(ac == ac_fast), TRUE)
 
   ## for K = 3
   clusters <- rep(1:3, 5)
   features <- matrix(rnorm(30), ncol = 2)
-  ac <- anticlustering(features, K = clusters, objective = variance_objective)
-  ac_fast <- fast_anticlustering(features, clusters)
+  ac <- anticlustering(features, K = clusters, objective = "variance")
+  ac_fast <- fast_anticlustering(features, clusters, exchange_partners = rep(list(1:15), 15))
   expect_equal(all(ac == ac_fast), TRUE)
 
   ## for K = 4
   clusters <- rep(1:4, 5)
   features <- matrix(rnorm(20), ncol = 1)
-  ac <- anticlustering(features, K = clusters, objective = variance_objective)
-  ac_fast <- fast_anticlustering(features, clusters)
+  ac <- anticlustering(features, K = clusters, objective = "variance")
+  ac_fast <- fast_anticlustering(features, clusters, exchange_partners = rep(list(1:20), 20))
   expect_equal(all(ac == ac_fast), TRUE)
 })
 
@@ -76,14 +76,14 @@ test_that("same results for local updating and recomputing - different group siz
   M <- 2
   clusters <- rep(1:3, c(30, 15, 15))
   features <- matrix(rnorm(N * M), ncol = M)
-  ac1 <- anticlustering(features, K = clusters, objective = variance_objective)
-  ac2 <- fast_anticlustering(features, clusters)
+  ac1 <- anticlustering(features, K = clusters, objective = "variance")
+  ac2 <- fast_anticlustering(features, clusters, exchange_partners = rep(list(1:N), N))
   ac3 <- anticlustering(features, clusters, objective = "variance")
-  expect_equal(all(ac1 == ac2), TRUE)
-  expect_equal(all(ac2 == ac3), TRUE)
+  expect_true(all(ac1 == ac2))
+  expect_true(all(ac2 == ac3))
   
   ac4 <- anticlustering(features, K = clusters, objective = diversity_objective)
   ac5 <- anticlustering(features, clusters)
-  expect_equal(all(ac4 == ac5), TRUE)
+  expect_true(all(ac4 == ac5))
   
-})
\ No newline at end of file
+})
